% my $p = shift;
<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge" />
    <!-- #VERSION# / #DATE# -->
    <link rel="SHORTCUT ICON" href="resource/<%= $p->{name} %>/favicon.ico"/>
    <title> <%= $p->{class} %> </title>
</head>
<body>
<div style="text-align: center; font-size: 30pt; right-margin: auto; margin-top: 100px">loading <%= $p->{class} %> ...</div>
<script type="text/javascript" src="script/<%= $p->{name} %>.js?v=#VERSION#"></script>
<iframe src="login" style="width:0px;height:0px;border:0px;" id="cbLoginIframe"></iframe>

</body>
</html>
